/*
 * Copyright 2005-2009 WSO2, Inc. http://wso2.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function getGadgetsJSON(urls, callback) {
    if (!urls || !urls.length)
        return;

    var json = '{ "context":{"language" : "en", "country": "US", "ignoreCache": "false"}, "gadgets" : [';

    for (var i = 0; i < urls.length; i++) {
        if (i == 0) {
            json += '{"moduleId" : 0, "prefs":{}, "url": "' + urls[i] + '"}';
        } else {
            json += ',{"moduleId" : 0, "prefs":{}, "url": "' + urls[i] + '"}';
        }
    }
    json += ']}';

    var metaDataService = '../../gadgets/metadata';

    var xmlHttpReq = createXmlHttpRequest();

    //Make sure the XMLHttpRequest object was instantiated
    if (xmlHttpReq)
    {
        // This is a synchronous POST, hence UI blocking. 
        xmlHttpReq.open("POST", metaDataService, false);

        //Send the proper header information along with the request
        xmlHttpReq.setRequestHeader("Content-type", "multipart/form-data");
        xmlHttpReq.setRequestHeader("Content-length", json.length);
        xmlHttpReq.setRequestHeader("Connection", "close");

        xmlHttpReq.send(json);
        var response = "";
        try {
            response = JSON.parse(xmlHttpReq.responseText);
        } catch(ex) {
        }
        return response;

    }
}

function createXmlHttpRequest() {
    var request;

    // Lets try using ActiveX to instantiate the XMLHttpRequest object
    try {
        request = new ActiveXObject("Microsoft.XMLHTTP");
    } catch(ex1) {
        try {
            request = new ActiveXObject("Msxml2.XMLHTTP");
        } catch(ex2) {
            request = null;
        }
    }

    // If the previous didn't work, lets check if the browser natively support XMLHttpRequest
    if (!request && typeof XMLHttpRequest != "undefined") {
        //The browser does, so lets instantiate the object
        request = new XMLHttpRequest();
    }

    return request;
}

function generateUniqueId()
{
    var d = new Date();
    return d.getTime();
}

function showAddGadgetsPage(gadgetGrp) {
    var grp = '';
    if (gadgetserver.gadgetLayout != null && gadgetserver.gadgetLayout != 'NA') {
        if (gadgetserver.gadgetLayout.length % 3 == 0) {
            grp = 'G1#';
        } else if (gadgetserver.gadgetLayout.length % 3 == 1) {
            grp = 'G2#';
        } else if (gadgetserver.gadgetLayout.length % 3 == 2) {
            grp = 'G3#';
        }
    } else {
        // This is an empty tab. Add gadgets to group 1
        grp = 'G1#';
    }
    window.location = "add-gadgets.jsp?tab=" + currentActiveTab + "&name=" + dashboardName + "&grp=" + grp;
}

/**
 * Goes through a given array of URLs and prefixes the relative URLs with the server's HTTP root
 * @param urls
 */
function sanitizeUrls(urls) {
    var sanitizedUrls = new Array();

    for (var x = 0; x < urls.length; x++) {
        var currentUrl = urls[x];
        if (currentUrl.charAt(0) == "/") {
            currentUrl = httpServerRoot + currentUrl;
        }
        sanitizedUrls.push(currentUrl);
    }

    return sanitizedUrls;
}

function removeCarriageReturns(string) {
    return string.replace(/\n/g, "");
}

/**
 * This is a silly implementation that will need to be overriden by almost all
 * real containers.
 * TODO: Find a better default for this function
 *
 * @param view The view name to get the url for
 */
function getUrlForView(view) {
    if (view === 'canvas') {
        return '/canvas';
    } else if (view === 'profile') {
        return '/profile';
    } else {
        return null;
    }
}

function getGadgetIdFromModuleId(moduleId) {
    // Quick hack to extract the gadget id from module id
    return parseInt(moduleId.match(/_([0-9]+)$/)[1], 10);
}

function showInputDialog(message, handleOk, handleCancel, closeCallback) {
    var strInput = "<div style='margin:20px;'><p>" + message + "</p><br/>" +
                   "<input type='text' id='carbon-ui-dialog-input' size='40' name='carbon-dialog-inputval'></div>";
    var strDialog = "<div id='dialog' title='WSO2 Carbon'>" + strInput + "</div>";

    jQuery("#dcontainer").html(strDialog);
    $("#dialog").dialog({
        close:function() {
            jQuery(this).dialog('destroy').remove();
            jQuery("#dcontainer").empty();
            if (closeCallback && typeof closeCallback == "function") {
                closeCallback();
            }
            return false;
        },
        buttons:{
            "OK":function() {
                var inputVal = document.getElementById("carbon-ui-dialog-input").value;
                handleOk(inputVal);
                jQuery(this).dialog("destroy").remove();
                jQuery("#dcontainer").empty();
                return false;
            },
            "Cancel":function() {
                jQuery(this).dialog("destroy").remove();
                jQuery("#dcontainer").empty();
                handleCancel();
            }
        },
        height:160,
        width:450,
        minHeight:160,
        minWidth:330,
        modal:true
    });
}

function confirmDialog(message, handleYes, gId) {
    /* This function always assume that your second parameter is handleYes function and third parameter is handleNo function.
     * If you are not going to provide handleYes function and want to give handleNo callback please pass null as the second
     * parameter.
     */
    var strDialog = "<div id='dialog' title='WSO2 Carbon'><div id='messagebox-confirm'><p>" +
                    message + "</p></div></div>";

    handleYes = handleYes || function() {
        return true
    };

    jQuery("#dcontainer").html(strDialog);

    jQuery("#dialog").dialog({
        close:function() {
            jQuery(this).dialog('destroy').remove();
            jQuery("#dcontainer").empty();
            return false;
        },
        buttons:{
            "Yes":function() {
                jQuery(this).dialog("destroy").remove();
                jQuery("#dcontainer").empty();
                handleYes(gId);
            },
            "No":function() {
                jQuery(this).dialog("destroy").remove();
                jQuery("#dcontainer").empty();
                return false;
            }
        },
        height:160,
        width:450,
        minHeight:160,
        minWidth:330,
        modal:true
    });
    return false;

}

function removeHTMLTags(inputItem) {
    inputItem = inputItem.replace(/&(lt|gt);/g, function (strMatch, p1) {
        return (p1 == "lt") ? "<" : ">";
    });
    var inputItemText = inputItem.replace(/<\/?[^>]+(>|$)/g, "");
    return inputItemText;
}

function fixBreadCrumb(dashboardName) {
    var breadCrumbDiv = document.getElementById("breadcrumb-div");
    var breadCrumbLinks = breadCrumbDiv.getElementsByTagName("a");

    for (var x = 0; x < breadCrumbLinks.length; x++) {
        var linkContent = breadCrumbLinks[x].href;
        if (linkContent.search(/dashboard/i) > 0) {
            breadCrumbLinks[x].href = linkContent + "&name=" + dashboardName;
        }
    }
}

function getServerBaseUrl(){
    var currentUrl = document.location.href;

    // We need to break the URL upto the last /carbon
    return currentUrl.substring(0, currentUrl.lastIndexOf("carbon"));
}
