var UserPrefsService = UserPrefsService || {};
//Creates a new cookie
UserPrefsService.createCookie = function createCookie(name, value, days) {
    if (days) {
        var date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        var expires = "; expires=" + date.toGMTString();
    } else
        var expires = "";
    document.cookie = name + "=" + value + expires + "; path=/";
}

//Reads the cookie
UserPrefsService.readCookie = function readCookie(name) {
    var nameEQ = name + "=";
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ')
            c = c.substring(1, c.length);
        if (c.indexOf(nameEQ) == 0)
            return c.substring(nameEQ.length, c.length);
    }
    return "NA";
}

//erase the cookie
UserPrefsService.eraseCookie = function eraseCookie(name) {
    UserPrefsService.createCookie(name, "", -1);
}

//The gadget counter
UserPrefsService.incrementGadgetCount = function incrementGadgetCount() {
    var gadgetId = UserPrefsService.readCookie("nextGadgetId");
    var nextGadgetId = Number(gadgetId) + 1;
    UserPrefsService.createCookie("nextGadgetId", nextGadgetId, 0);
}

//The tab counter
UserPrefsService.incrementTabCount = function incrementGadgetCount() {
    var gadgetId = UserPrefsService.readCookie("nextTabId");
    var nextGadgetId = Number(gadgetId) + 1;
    UserPrefsService.createCookie("nextTabId", nextGadgetId, 0);
}

/**
 * Adds new gadgets. the new gadgets are stored in cookies
 * every gadget has a unique id a combination of "gadget" + next_available_gadgetId
 * Gadget layout is saved containing the tab id in it's name
 * the values in the layout are the variable gadget ID
 */
UserPrefsService.addGadget = function addGadget(userId, tabId, url,
                                                dashboardName, group) {
    var curGadgetId = UserPrefsService.readCookie("nextGadgetId");
    if (curGadgetId == "NA") {
        curGadgetId = 0;
        var nextGadgetId = curGadgetId + 1;
        UserPrefsService.createCookie("nextGadgetId", nextGadgetId, 0);
    } else {
        UserPrefsService.incrementGadgetCount();
    }

    UserPrefsService.jsonBuilder("gadget-" + curGadgetId, url);
    //UserPrefsService.createCookie("gadget-" + curGadgetId, url, 0);

    var gadgetLayout = UserPrefsService.readCookie("gadgetLayout-" + tabId);
    if (gadgetLayout == "NA") {
        gadgetLayout = group + curGadgetId;
        UserPrefsService.createCookie("gadgetLayout-" + tabId, gadgetLayout, 0);
    } else {
        gadgetLayout = gadgetLayout + "," + group + curGadgetId;
        UserPrefsService.createCookie("gadgetLayout-" + tabId, gadgetLayout, 0);
    }

    return true;
}
/**
 * Adds new tab to the layout
 * @param userId
 * @param tabTitle
 * @param dashboardName
 * @return current tab id
 */
UserPrefsService.addNewTab = function addNewTab(userId, tabTitle, dashboardName) {
    if (UserPrefsService.hasTabName(userId, tabTitle, dashboardName)) {
        return 0;
    }
    var curTabId = UserPrefsService.readCookie("nextTabId");
    if (curTabId == "NA") {
        curTabId = 1;
        var nextTabId = curTabId + 1;
        UserPrefsService.createCookie("nextTabId", nextTabId, 0);
    } else {
        UserPrefsService.incrementTabCount();
    }

    if (tabTitle == "") {
        tabTitle = "tab " + curTabId;
    }

    UserPrefsService.createCookie("tab-" + curTabId, tabTitle, 0);

    var tabLayout = UserPrefsService.readCookie("tabLayout");
    if (tabLayout == "NA") {
        tabLayout = curTabId;
        UserPrefsService.createCookie("tabLayout", tabLayout, 0);
    } else {
        tabLayout = tabLayout + "," + curTabId;
        UserPrefsService.createCookie("tabLayout", tabLayout, 0);
    }

    return curTabId;
}

UserPrefsService.getGadgetLayout = function getGadgetLayout(userId, tabId,
                                                            dashboardName) {
    var layoutArray = UserPrefsService.readCookie("gadgetLayout-" + tabId)
    if (layoutArray == '') {
        layoutArray = 'NA';
    }
    return layoutArray;
}

UserPrefsService.getGadgetPrefs = function getGadgetPrefs(userId, gadgetId,
                                                          prefId, dashboardName) {
    response = UserPrefsService.readCookie("gadgetPref-" + gadgetId + "-" + prefId);
    return response;

}

UserPrefsService.getGadgetUrlsToLayout = function getGadgetUrlsToLayout(userId,
                                                                        tabId, dashboardName) {
    var gadgetIdGrpArray = UserPrefsService.getGadgetLayout("", tabId, "").split(",");
    if (gadgetIdGrpArray == 'NA' || gadgetIdGrpArray == '') {
        return 'NA';
    }
    var gadgetIdArrayStr = '';
    for (var x = 0; x < gadgetIdGrpArray.length; x++) {
        if (gadgetIdArrayStr == '') {
            //Gadget layout return with its group id; it's been split and Id is taken
            gadgetIdArrayStr = gadgetIdGrpArray[x].split('#')[1];
        } else {
            //Gadget layout return with its group id; it's been split and Id is taken
            gadgetIdArrayStr = gadgetIdArrayStr + ',' + gadgetIdGrpArray[x].split('#')[1];
        }
    }

    var gadgetIdArray = gadgetIdArrayStr.split(',');

    var urlArray = new Array();
    for (var i = 0; i < gadgetIdArray.length; i++) {
        //urlArray[i] = UserPrefsService.readCookie("gadget-" + gadgetIdArray[i]);
        urlArray[i] = UserPrefsService.getGadgetUrl("gadget-" + gadgetIdArray[i]);
    }

    return urlArray;
}

UserPrefsService.getTabLayout = function getTabLayout(userId, dashboardName) {
    var response = UserPrefsService.readCookie("tabLayout");
    if (response == "NA") {
        UserPrefsService.createCookie("tabLayout", "0", 0);
        return "0";
    }
    return response;
}

UserPrefsService.getTabTitle = function getTabTitle(userId, tabId,
                                                    dashboardName) {
    if (tabId == 0) {
        return "Home"
    }
    var response = UserPrefsService.readCookie("tab-" + tabId);
    return response;
}

UserPrefsService.removeGadget = function removeGadget(userId, tabId, gadgetId,
                                                      dashboardName) {
    var curLayoutArray = UserPrefsService.getGadgetLayout("", tabId, "").split(",");
    var newLayout = "";
    for (var i = 0; i < curLayoutArray.length; i++) {
        if (curLayoutArray[i].split('#')[1] != gadgetId) {
            if (newLayout == "") {
                newLayout = curLayoutArray[i];
            } else {
                newLayout = newLayout + "," + curLayoutArray[i];
            }
        }
    }

    UserPrefsService.setGadgetLayout("", tabId, newLayout, "");

    UserPrefsService.eraseCookie("gadget-" + gadgetId);

    return true;
}

UserPrefsService.removeTab = function removeTab(userId, tabId, dashboardName) {
    var curTabLayoutArray = UserPrefsService.getTabLayout("", "").split(",");
    var newLayout = "";
    for (var i = 0; i < curTabLayoutArray.length; i++) {
        if (curTabLayoutArray[i] != tabId) {
            if (newLayout == "") {
                newLayout = curTabLayoutArray[i];
            } else {
                newLayout = newLayout + "," + curTabLayoutArray[i];
            }
        }
    }

    UserPrefsService.createCookie("tabLayout", newLayout, 0);

    UserPrefsService.eraseCookie("tab-" + tabId);
    return true;
}

UserPrefsService.setGadgetLayout = function setGadgetLayout(userId, tabId,
                                                            newLayout, dashboardName) {
    UserPrefsService.createCookie("gadgetLayout-" + tabId, newLayout, 0);
    return true;
}

UserPrefsService.setGadgetPrefs = function setGadgetPrefs(userId, gadgetId,
                                                          prefId, value, dashboardName) {
    UserPrefsService.createCookie("gadgetPref-" + gadgetId + "-" + prefId, value, 0);
    return true;
}


/*UserPrefsService.duplicateTab = function duplicateTab(userId, dashboardName, sourceTabId, newTabName)
 {
 // Getting the Gadget Layout of the source Tab
 var sourceLayout = UserPrefsService.getGadgetLayout(userId, sourceTabId, dashboardName);

 // Creating a new Tab
 var newTabId = UserPrefsService.addNewTab(userId, newTabName, dashboardName);

 // Setting the layout of the new Tab using the source tab's layout
 if (UserPrefsService.setGadgetLayout(userId, newTabId, sourceLayout, dashboardName)) {
 return parseInt(newTabId);
 }
 }*/

UserPrefsService.duplicateTab = function duplicateTab(userId, dashboardName, sourceTabId, newTabName)
{
    // Getting the Gadget Layout of the source Tab
    var sourceLayout = UserPrefsService.getGadgetLayout(userId, sourceTabId, dashboardName);

    var sourceGadgets = sourceLayout.split(',');

    // Creating a new Tab
    var newTabId = UserPrefsService.addNewTab(userId, newTabName, dashboardName);

    for (var i = 0; i < sourceGadgets.length; i++) {
        //var gUrl = UserPrefsService.readCookie("gadget-" + sourceGadgets[i].split('#')[1]);
        var gUrl = UserPrefsService.getGadgetUrl("gadget-" + sourceGadgets[i].split('#')[1]);
        UserPrefsService.addGadget(null, newTabId, gUrl, null, sourceGadgets[i].split('#')[0] + '#');
    }

    return parseInt(newTabId);

    // Setting the layout of the new Tab using the source tab's layout
    /*if (UserPrefsService.setGadgetLayout(userId, newTabId, sourceLayout, dashboardName)) {
     return parseInt(newTabId);
     }*/
}

UserPrefsService.copyGadget = function copyGadget(userId, tab, dashboardName, sourceGadgetId) {
    UserPrefsService.jsonBuilder();
    //takeing the gadget url from cookie
    //var gadgetUrl = UserPrefsService.readCookie("gadget-" + sourceGadgetId);
    var gadgetUrl = UserPrefsService.getGadgetUrl("gadget-" + sourceGadgetId);
    UserPrefsService.addGadget(null, tab, gadgetUrl, null, "G1#");

}

UserPrefsService.moveGadgetToTab = function moveGadgetToTab(userId, tab, dashboardName, sourceGadgetId) {

    //takeing the gadget url from cookie
    //var gadgetUrl = UserPrefsService.readCookie("gadget-" + sourceGadgetId);
    var gadgetUrl = UserPrefsService.getGadgetUrl("gadget-" + sourceGadgetId);
    UserPrefsService.addGadget(null, tab, gadgetUrl, null, "G1#");

}

UserPrefsService.hasTabName = function hasTabName(userId, tabName, dashboardName) {

    //takeing the gadget url from cookie
    var tabIds = UserPrefsService.getTabLayout(userId, dashboardName).split(',');
    for (var x = 0; x < tabIds.length; x++) {
        var tabTitle = UserPrefsService.getTabTitle(userId, tabIds[x], dashboardName);
        if (tabName == tabTitle) {
            return true;
        }
    }
    return false;
}

UserPrefsService.isNewUser = function isNewUser() {
    var curTabId = UserPrefsService.readCookie("nextGadgetId");
    return (curTabId == "NA");
}

UserPrefsService.isValidTab = function isValidTab(userId, currentActiveTab, dashboardName) {
    var title = UserPrefsService.getTabTitle(userId, currentActiveTab, dashboardName);
    if (title == 'NA') {
        return false;
    }

    return true;
}

UserPrefsService.jsonBuilder = function jsonBuilder(gId, gUrl) {
    var gadget = {};
    gadget['gUrl'] = gUrl;
    var gadgets = UserPrefsService.getGadgetJson();
    if (gadgets == null) {
        gadgets = {};
    }
    gadgets[gId] = gadget;
    UserPrefsService.saveGadgetJson(gadgets);
}

UserPrefsService.saveGadgetJson = function saveGadgetJson(gJson) {
    UserPrefsService.createCookie("gadgetJson", JSON.stringify(gJson), 0);
}

UserPrefsService.getGadgetJson = function getGadgetJson() {
    var gadgetJsonText = UserPrefsService.readCookie("gadgetJson");
    if (gadgetJsonText == 'NA') {
        return null;
    } else {
        return JSON.parse(gadgetJsonText);
    }
}

UserPrefsService.getGadgetUrl = function getGadgetUrl(gId) {
    var gadgetJsonObj = UserPrefsService.getGadgetJson();
    return gadgetJsonObj[gId].gUrl;
}